"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ret = __importStar(require("retry"));
const constants_1 = require("../constants");
const delay_1 = __importDefault(require("./delay"));
/**
 * Attempt a promise-based operation with retries built in.
 *
 * @param operation     Operation to retry. It is passed the zero-indexed
 *                      attempt number and whether this is the last try.
 * @param shouldRetry   Function that takes the error that was thrown and
 *                      determines whether it should be retried. Defaults to
 *                      always retry.
 * @param options       Configuration options for attempts and timeouts from the
 *                      `retry` library.
 */
async function retry(operation, shouldRetry = () => true, options = constants_1.INTERNAL_RETRY_OPTIONS) {
    const delays = ret.timeouts(options);
    for (const [index, timeout] of delays.entries()) {
        try {
            // Attempt the operation
            return await operation(index, false);
        }
        catch (err) {
            if (shouldRetry(err)) {
                // Wait the appropriate time before the next attempt if we
                // should retry
                await (0, delay_1.default)(timeout);
            }
            else {
                // If we shouldn't retry, throw the error we received
                throw err;
            }
        }
    }
    // Last try
    return operation(delays.length, true);
}
exports.default = retry;
//# sourceMappingURL=retry.js.map